/*
         xdipo copyright Jan Panteltje 2005 and later.

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/

                                  /* SET TABS TO 4 TO READ THIS SOURCE */

#include "xdipo.h"

FD_xdipo *fdui;

FD_xdipo *fd_main_form;



int goto_angular_position(char *position)
{
int a;
double degrees;
char direction[10];
int nib1h, nib1l, nib2h, nib2l;
double da;
int temp[] = {0xe0, 0x31, 0x6e, 0x00, 0x00};

if(debug_flag)
	{
	fprintf(stderr, "goto_angular_position(): arg position=%s\n", position);
	}

/* argument check */
if(! position) return 0;

a = sscanf(position, "%lf %1[EW]s", &degrees, direction);
if(a != 2)
	{
	if(gui_flag)	
		{
		fl_show_alert("Usage 23.12 E or 76.93 W", "space required", "", 0);

		return 0;
		}
	else
		{
		fprintf(stderr, "xdipo: goto_angular_position(): usage 23.12 E or 76.93 W, space required\n");

		return 0;
		}
	}


/*
From:
 http://www.satelliteguys.us/free-air-fta-discussion/163571-regarding-goto-angular-position-command-diseqc.html
You're using the wrong command set... those are for terrestrial antenna
rotators. The correct commands are E0 31 6E D1 40 for 20W and E0 31 6E E1
40 for 20E. The last nibble is the fractional degree setting, which is in
units of 1/16 (the DiSEqC spec explains this almost correctly, but leaves
out some valid values such as 4h and Fh).
These commands aren't documented publicly (until now, I guess ). I found
them by intuition alone. They are probably documented in the USALS spec, but
I've never seen it so there may even more that I'm unaware of.
*/

if( toupper(direction[0]) == 'W')
	{
//	nib1h = 0; // old code
	nib1h = 0xd; // this is necessary for dish rotors (as opposed to terrestrial rotors) patch by Garnet MacPhee
	}
else
	{
//	nib1h = 0xe; // experimentally, maybe this is only valid for the Moteck Sl 2100A ? old code
	nib1h = 0xe; // this is necessary for dish rotors (as opposed to terrestrial rotors) patch by Garnet MacPhee
	}

nib1l = degrees / 16;
nib2h = degrees - (16 * nib1l);

da = degrees - (16 * nib1l) - nib2h;
nib2l = da * 16.0;

/* doing it again rounding to the nearest integer, more conform diseqc spec */
//fprintf(stderr, "OLD: nib2l=%02x\n", nib2l);
double nearbyint (double X); /* should be declared in math.h */
nib2l = nearbyint( (da * 16.0) );

if(debug_flag)
	{
	fprintf(stderr, "goto_angular_position(): da=%.2f nib2l=%02x\n", da, nib2l);
	}

/* would generate a carry to 0x10 for values close to one of da, because of rounding up to 1 */	
if( (da > 0.9) && ( (nib2l & 0xf) == 0) ) nib2l = 0xf;


temp[3] = ( (nib1h & 0xf) << 4) | (nib1l & 0xf);
temp[4] = ( (nib2h & 0xf) << 4) | (nib2l & 0xf);


if(debug_flag)
	{
	fprintf(stderr, "goto_angular_position(): calculated command=%02x %02x %02x %02x %02x\n", temp[0], temp[1], temp[2], temp[3], temp[4]);

	fprintf(stderr, "\n");
	}

//return 1;


a = send_diseqc_command(temp, 5);
if(! a) return 0;

return 1;
} /* end function goto_angular_position */



int print_usage()
{
fprintf(stderr,
"\n                      xdipo-%s copyright 2005 and later Jan Panteltje\n", XDIPO_VERSION);
fprintf(stderr, "\
Usage:\n\
xdipo [-a pid1 ...pid16] [-b satellite_nr] [-c seconds] [-g \"angle E|W\"] [-f frequency] [-h] [-l logfile]\n\
 [-o] [-p polarization] [-q] [-s symbolrate] [-v] [-w] [-x demux_device] [-s symbolrate] [-v] [-w]\n\
 [-x demux_device] [-y frontend_device] [-z dvr_device]\n\n\
-a \"pid1 pid2... pid16\" up to 16 pids (default is 8192, that passes all pids), or\n\
-a \"s serviceID\"         first a 's' then a space, and then the service ID of the station,\n\
-b satellite_nr  a         goto satellite number and exit. No GUI, no tuning.\n\
-c seconds                seconds to wait while moving dish before testing signal lock, default is 3.\n\
-d S|S2                   delivery system, S for DVB-S or S2 for DVB-S2, default is S.\n\
-e fec                    FEC, either 1/2 2/3 3/4 3/5 4/5 5/6 6/7 8/9 9/10, or AUTO, default is AUTO.\n\
-f frequency              MHz float (12092.0).\n\
-g \"position\"             \"13.2 E\" or \"19.2 W\" goto angle in degrees and exit. No GUI, no tuning,\n\
                           space between angle and E or W is required, so are the quotes.\n\
-h                        help (this help).\n\
-l path_filename          log bit errors to filename in in script mode, and only when using -o\n\
                           (use /dev/fd/2 for stderr, do NOT use /dev/fd/1 (=stdout) when using -o!).\n\
-m modulation             modulation system, either BPSK QPSK or 8PSK. default is BPSK.\n\
-o                        goto sat as indicated by -b or -g, tune to transponder as indicated by -f -p -v,\n\
                           and write data to stdout. No GUI.\n\
-p polarization           h or v (h).\n\
-q                        send stop command and exit. No GUI, no tuning.\n\
-r rolloff                rolloff, either 0.35 0.25 0.20 or AUTO, default AUTO.\n\
-s symbolrate             (27500).\n\
-v                        debug mode, prints functions and arguments.\n\
-w                        power down, set LNB voltage to zero, 22kHz tone off and exit. No GUI.\n\
-x demux_device_name      default /dev/dvb/adapter0/demux0\n\
-y frontend_device_name   default /dev/dvb/adapter0/frontend0\n\
-z dvr_device_name        default /dev/dvb/adapter0/dvr0\n\
");

return 1;
} /* end function print_usage */


int main(int argc, char **argv)
{
int a = 0;
int b = 0;
char temp[4096];
char temp2[4096];
struct passwd *userinfo;
FL_IOPT fl_iopt;
Pixmap p, mask;
unsigned w, h;
FILE *fptr;
char position[80];
int halt_flag, pos_flag, sat_flag, power_flag, out_flag;
static int bit_error_rate;
char *ptr;

/* defaults */
debug_flag = 0;
gui_flag = 0;

/* get user info */
userinfo = getpwuid(getuid() );

/* get home directory */ 
home_dir = strsave(userinfo -> pw_dir);
if(! home_dir)
	{
	fprintf(stderr, "xdipo: main(): could allocate space for home_dir, aborting.\n");

	exit(1);
	}

user_name = strsave(userinfo -> pw_name);
if(! user_name)
	{
	fprintf(stderr, "xdipo: main(): could allocate space for user_name, aborting.\n");
	exit(1);
	}

if(strcmp(user_name, "root") == 0) root_flag = 1;

frontend_device_name = strsave("/dev/dvb/adapter0/frontend0");
if(! frontend_device_name)
	{
	fprintf(stderr,\
	"xdipo: main(): could not allocate space for frontend_device_name, aborting.\n");

	exit(1);
	}

demux_device_name = strsave("/dev/dvb/adapter0/demux0");
if(! demux_device_name)
	{
	fprintf(stderr,\
	"xdipo: main(): could not allocate space for demux_device_name, aborting.\n");

	exit(1);
	}

dvr_device_name = strsave("/dev/dvb/adapter0/dvr0");
if(! dvr_device_name)
	{
	fprintf(stderr,\
	"xdipo: main(): could not allocate space for dvr_device_name, aborting.\n");

	exit(1);
	}

/* try to get previous output dir */
sprintf(temp, "%s/.xdipo/output_dir", home_dir);
fptr = fopen(temp, "r");
if(fptr)
	{
//	a = fscanf(fptr, "%as", &output_dir); 
	a = fscanf(fptr, "%s", temp2); 
	if(a != 1)
		{
		fprintf(stderr, "xdipo: main(): file format error in %s, a=%d\n", temp, a);
		}	

	output_dir = strsave(temp2);
	if(!output_dir)
		{
		fprintf(stderr, "xdipo: main(): could allocate space for output_dir from temp2, aborting.\n");

		exit(1);
		}

	fclose(fptr);
	}
else
	{
	/* set default output directory */
	output_dir = strsave(home_dir);
	if(! output_dir)
		{
		fprintf(stderr, "xdipo: main(): could allocate space for output_dir, aborting.\n");

		exit(1);
		}
	}

/* pass the whole ts */
strcpy(pids_string, "8192");

bit_errors_log_filename = 0;
satellite_number = 0;
frequency = 12092.0;
polarization = 'h';
symbolrate = 27500;
angle = -13.0;
goto_mode = GOTO_ANGLE;
strcpy(position, "13 E");
nudge_size = 10;

halt_flag = 0;
pos_flag = 0;
sat_flag = 0;
power_flag = 0;
out_flag = 0;
seconds_to_wait = 3;

dsp_beep_flag = 0;

/* for S2 version */
strcpy(type, "S");
strcpy(modulation, "BPSK");
strcpy(fec, "AUTO");
rolloff = 0.0;

process_packets = PROCESS_PACKETS;

/* end defaults */

while(1)
	{
	a = getopt(argc, argv, "a:b:c:d:e:g:f:hl:m:op:qr:s:vwx:y:z:");
	if(a == -1) break;

	switch(a)
		{
		case 'a':
			strncpy(pids_string, optarg, 79);
			break;
		case 'b': // goto satellite number
			if(pos_flag)
				{
				fprintf(stderr,\
"xdipo: command line error:'-b %s' you already specified an angular position with '-g %s', aborting.\n",\
				optarg, position);

				exit(1);
				}

			satellite_number = atoi(optarg);
			sat_flag = 1;
			break;
		case 'c': // seconds to wait
			seconds_to_wait = atoi(optarg);
			break;
		case 'd': // delivery system
			a = 1;
			if(strcmp(optarg, "S") == 0) a = 0;
			if(strcmp(optarg, "S2") == 0) a = 0;
			if(a)
				{
				print_usage();

				fprintf(stderr, "xdipo: main(): invalid delivery system, aborting.\n");
				
				exit(1);
				}

			strncpy(type, optarg, 2); 
			break;
		case 'e': // fec
			a = 1;
			if(strcasecmp(optarg, "AUTO") == 0) a = 0;
			if(strcmp(optarg, "1/2") == 0) a = 0;
			if(strcmp(optarg, "2/3") == 0) a = 0;
			if(strcmp(optarg, "3/4") == 0) a = 0;
			if(strcmp(optarg, "3/5") == 0) a = 0;
			if(strcmp(optarg, "4/5") == 0) a = 0;
			if(strcmp(optarg, "5/6") == 0) a = 0;
			if(strcmp(optarg, "6/7") == 0) a = 0;
			if(strcmp(optarg, "7/8") == 0) a = 0;
			if(strcmp(optarg, "8/9") == 0) a = 0;
			if(strcmp(optarg, "9/10") == 0) a = 0; 
			if(a)
				{
				print_usage();
				
				fprintf(stderr, "xdipo: main(): invalide fec, aborting.\n");
				
				exit(1);
				}
			
			strncpy(fec, optarg, 4);
			break;
		case 'h':	// help
			print_usage();
			exit(1);
			break;
		case 'f':	// frequency
			frequency = atof(optarg);
			break;
		case 'g':	// goto position
			if(sat_flag)
				{
				fprintf(stderr,\
				"xdipo: command line error: '-g %s' you already specified a satellite number with '-b %d', aborting.\n",\
				optarg, satellite_number);

				exit(1);
				}

			strncpy(position, optarg, 20);
			pos_flag = 1;

			break;
		case 'l':
			bit_errors_log_filename = strsave(optarg);
			if(! bit_errors_log_filename)
				{
				fprintf(stderr,\
				"xdipo: main(): could not allocate space for bit_errors_log_filename\n");
				}
			break;
		case 'm': // modulation system
			a = 1;
			if(strcmp(optarg, "BPSK") == 0) a = 0;
			if(strcmp(optarg, "QPSK") == 0) a = 0;
			if(strcmp(optarg, "8PSK") == 0) a = 0;
			if(a)
				{
				print_usage();

				fprintf(stderr, "xdipo: main(): invalid modulation system, aborting.\n");
				
				exit(1);
				}

			strncpy(modulation, optarg, 8);
			break;
		case 'o': // write from dvr device to output
			out_flag = 1;
			break;
		case 'p': // polarization
			polarization = optarg[0];
			break;
		case 'q': // halt and exit
			halt_flag = 1;
			break;
		case 'r': // rolloff
			rolloff = atof(optarg);
			a = 1;
			if(rolloff == 0.0) a = 0;
			if(rolloff == 0.20) a = 0;
			if(rolloff == 0.25) a = 0;
			if(rolloff == 0.35) a =0;
			if(a)
				{
				print_usage();	

				fprintf(stderr, "xdipo: main(): invalid rolloff, aborting.\n");

				exit(1);
				}	

			break;
		case 's': // symbolrate
			symbolrate = atoi(optarg);
			break;
		case 'v': // debug
			debug_flag = 1;
			break;
		case 'w': // power to LNB off, 22kHz tone off.
			power_flag = 1;
			break;
		case 'x': // demux device name
			free(demux_device_name);
			demux_device_name = strsave(optarg);
			if(! demux_device_name)
				{
				fprintf(stderr,\
				"xdipo: main(): could not allocate space for demux_device_name, aborting.\n");

				exit(1);
				}
			break;
		case 'y': // frontend device name
			free(frontend_device_name);
			frontend_device_name = strsave(optarg);
			if(! frontend_device_name)
				{
				fprintf(stderr,\
				"xdipo: main(): could not allocate space for frontend_device_name\n");

				exit(1);
				}

			break;
		case 'z': // dvr device name
			free(dvr_device_name);
			dvr_device_name = strsave(optarg);
			if(! dvr_device_name)
				{
				fprintf(stderr,\
				"xdipo: main(): could not allocate space for dvr_device_name\n");

				exit(1);
				}

			break;
        case -1:
        	break;
		case '?':
			if (isprint(optopt) )
 				{
 				fprintf(stderr, "Unknown option `-%c'.\n", optopt);
 				}
			else
				{
				fprintf(stderr, "Unknown option character `\\x%x'.\n",\
				optopt);
				}
			print_usage();

			exit(1);
			break;			
		default:
			/* run in default mode if no arguments specified */
			break;
		} /* end switch a */
	} /* end while getopt() */

fd_frontend = open(frontend_device_name, O_RDWR | O_NONBLOCK);
if(fd_frontend < 0)
	{
	fprintf(stderr, "xdipo: main(): open frontend device %s failed, aborting.\n", frontend_device_name);
	perror("reason ");

	exit(1);
	}

if(debug_flag)
	{
	fprintf(stderr,\
	"satellite_number=%d position=%s frequency=%.2f polarization=%c symbolrate=%d\n",\
	satellite_number, position, frequency, polarization, symbolrate);
	fprintf(stderr, "frontend_device=%s\n", frontend_device_name);
	fprintf(stderr, "demux_device=%s\n", demux_device_name);
	fprintf(stderr, "dvr_device=%s\n", dvr_device_name);
	fprintf(stderr, "power_flag=%d halt_flag=%d pos_flag=%d sat_flag=%d out_flag=%d\n",\
	power_flag, halt_flag, pos_flag, sat_flag, out_flag);
	fprintf(stderr, "pids_string=%s\n", pids_string);
	fprintf(stderr, "seconds_to_wait=%d\n", seconds_to_wait);
	}


input_buffer = (uint8_t *)malloc(process_packets * 188);
if(! input_buffer)
	{
	fprintf(stderr, "jpinfo: could not allocate space for input_buffer, aborting.\n");

	exit(1);
	}

if(bit_errors_log_filename)
	{
	fpbe = fopen(bit_errors_log_filename, "w");
	if(! fpbe)
		{
		fprintf(stderr,\
		"xdipo: main(): could not open file %s for write, aborting.\n", bit_errors_log_filename);

		exit(1);
		}
	}

/* parse pids_string, sets globals use_filter_flag and service_id if a servce_id is specified in teh PID field */
if(! parse_pids_string(pids_string) )
	{
	fprintf(stderr, "could not parse pids_string, aborting.\n");

	exit(1);
	}

if(power_flag)
	{
	a = ioctl(fd_frontend, FE_SET_TONE, SEC_TONE_OFF);
	if(a)
		{
		fprintf(stderr, "xdipo: main(): set tone failed.\n");
		perror("reason ");
		}

	b = ioctl(fd_frontend, FE_SET_VOLTAGE, SEC_VOLTAGE_OFF);
	if(b)
		{
		fprintf(stderr, "xdipo: main(): set voltage failed.\n");
		perror("reason ");
		}

	if(a || b) exit(1);
	else exit(0);
	} /* end if power_flag */

if(halt_flag)
	{
	static int temp_halt[] = {0xe0, 0x31, 0x60};

	a = send_diseqc_command(temp_halt, 3);
	if(!a)
		{
		fprintf(stderr, "xdipo: main(): could not execute positioner motor halt command, aborting.\n");
		exit(1);
		}
	else exit(0);	
	} /* end if halt_flag */

if(pos_flag)
	{
	b = 0;
	while(1)
		{
		if(debug_flag)
			{
			fprintf(stderr, "main(): try %d position %s\n", b, position);
			}

		a = goto_angular_position(position);
		if(! a)
			{
			fprintf(stderr, "xdipo: main(): could not go to position %s, aborting.\n", position);

			exit(1);
			}

		fflush(NULL);

		/* wait requested seconds for dish to move */
		if(seconds_to_wait) sleep(seconds_to_wait);

		/* read frontend status */
		if(! get_frontend_status() )
			{
			fprintf(stderr, "xdipo(): get_frontend_status() failed, aborting.\n");
			
			exit(1);
			}

		if(lock_flag)
			{
			fprintf(stderr, "xdipo: HAVE LOCK\n");

			break;
			}

		b++;

		if(b == MAX_POSITIONER_TRIES)
			{
			fprintf(stderr,\
			"xdipo: main(): could not get lock on satellite after trying %d times to steer positioner to %s, aborting.\n",\
			b, position);

			exit(1);
			}
		} /* end while position disk */
	} /* end if pos_flag */

if(sat_flag)
	{
	int temp[] = {0xe0, 0x31, 0x6b, 0x00};

	temp[3] = satellite_number;

	a = send_diseqc_command(temp, 4);
	if(!a)
		{
		fprintf(stderr,\
		"xdipo: main(): could not command postioner to sat numer %d, aborting.\n",\
		satellite_number);

		exit(1);
		}
	} /* end if sat_flag */

if(! out_flag) // script mode, just position disk
	{
	if(pos_flag) exit(0);
	if(sat_flag) exit(0);
	}

/* use remote or not */
ptr = load_setting("enable_remote_mode");
if(ptr)
	{
	a = sscanf(ptr, "%d", &b);
	if(a == 1) enable_remote_flag = b;
	}

if(enable_remote_flag)
	{
	if(! start_remote() )
		{
		fprintf(stderr, "xdipo: main(): could not initilize remote thread, aborting.\n");
	
		exit(1);
		}
	} /* end if enable_remote_flag */

if(out_flag) // script mode tune, and /dev/dvr to stdout
	{
	static int dummy[] = {0x0};
	static char buffer[DVR_BUFFER_SIZE];

	/* tune to transponder */		
	a = send_diseqc_command(dummy, 0);
	if(!a)
		{
		fprintf(stderr, "xdipo: main(): send_diseqc_command() length zero failed, aborting.\n");
			
		exit(1);
		}

	/* open the dvr device for read */
	fd_dvr = open(dvr_device_name, O_RDONLY | O_NONBLOCK);
	if(fd_dvr < 0)
		{
		fprintf(stderr,\
		"xdipo: main(): could not open dvr device %s for read, aborting.\n", dvr_device_name);

		perror("reason ");

		exit(1);
		}

	a = ioctl(fd_dvr, DMX_SET_BUFFER_SIZE, DMX_BUFFER_SIZE);
	if(a  == -1)
		{
		fprintf(stderr, "xdipo: main(): could not set DMX buffer size, aborting.\n"); 
		
		perror("reason ");

		exit(1);
		}

	/* if service_id specified read fd_dvr for PAT 0 to find PMT and other service_id related pids */
	if(use_filter_flag)  /* set by parse_pids */
		{
		/* read fd_dvr for PAT 0 to find PMT and other service_id related pids */

		if(! process_stream(fd_dvr, service_id) )
			{
			fprintf(stderr, "xdipo: main(): could not find program related pids for service_id=%d, aborting.\n", service_id);

			exit(1);
			}

		} /* end if use_filter_flag */
		

	pfds[0].fd = fd_dvr;
	pfds[0].events = POLLIN | POLLPRI;

	/* read from dvr device and write to stdout */
	while(1)
		{
		poll(pfds, 1, 500);

		if(pfds[0].revents > 3)
			{
			fprintf(stderr, "WARN pfds[0].revents=%d\n", pfds[0].revents);
			}

		if( (pfds[0].revents & POLLIN) || (pfds[0].revents &  POLLPRI) )
			{
			a = read(fd_dvr, buffer, DVR_BUFFER_SIZE);
			if(a == -1)
				{
				if(errno == EAGAIN) continue;
				
				fprintf(stderr, "xdipo: main(): read fd_dvr failed\n");
				perror("reason ");

				if(errno == EOVERFLOW)
					{
					user_warning_dvr_buffer_size(gui_flag);
					}
				else
					{
					fprintf(stderr, "xdipo: main(): WARNING: Unknown error reading from dvr device\n");
//					exit(1);
					}
				}

			b = fwrite(buffer, sizeof(char), a, stdout);
			if(b != a)
				{
				fprintf(stderr, "xdipo: main(): WARNING: fwrite failed\n");
				perror("reason ");

				exit(1);
				}

			} /* end if data */

		/*
		seems driver changed, for BER, now aborts if read.
		frontend.h:#define FE_READ_BER                    _IOW('o',65,uint32_t *) number has changed
		works fine if out of loop when no data.. possibly too slow, also with NON_BLOCK.
		*/
		if(bit_errors_log_filename)
			{			
			/* bit error rate, multiple of 10-9, 2500 = 2.5 10-6 is 1 error in 400000 */
			a = ioctl(fd_frontend, FE_READ_BER, &bit_error_rate);
			if(a < 0)
				{
				fprintf(stderr, "xdipo: main(): error=%d reading fd_frontend\n", a);
				perror("reason ");

//				exit(1);
				}

			if(debug_flag)
				{
				fprintf(stderr, "xdipo: main(): bit_error_rate=%d\n", bit_error_rate);
				}

			if(bit_error_rate != 0)
				{
				if(fpbe)
					{
					fprintf(fpbe, "%s bit_errors=%d\n", get_universal_time_as_string(), bit_error_rate);
					fflush(fpbe);
					}
				}
			} /* end if bit_errors_log_filename */

		} /* end while read from dvr device and write to stdout */
	} /* end if out_flag */

/* never here in script mode */

/* to use 'beep' in GUI we need to be root */
if(root_flag) iopl(3);	

/* we will now be using the GUI, this enables fl_show_alert() etc.. */
gui_flag = 1;

/* against flashing */
/* set double buffering */
fl_iopt.doubleBuffer = 1;
fl_set_defaults(FL_PDDouble, &fl_iopt);
 
//fl_set_goodies_font(FL_BOLD_STYLE, FL_MEDIUM_SIZE);
 
fl_initialize(&argc, argv, 0, 0, 0);
fd_main_form = create_form_main_form();

/* read the icon */
sprintf(temp, "%s/.xdipo/xdipo.xpm", home_dir);
p = fl_read_pixmapfile(fl_root, temp, &w, &h, &mask, 0, 0, 0);
fl_set_form_icon(fdui -> main_form, p, mask);

/* show the main form */
sprintf(temp, "xdipo-%s", XDIPO_VERSION);
fl_show_form(fd_main_form -> main_form, FL_PLACE_FREE, FL_FULLBORDER, temp);

sprintf(temp, "%d", nudge_size);
fl_set_input(fdui -> main_form_step_value_input_field, temp);

sprintf(temp, "%d", satellite_number);
fl_set_input(fdui -> main_form_satellite_number_input_field, temp);

sprintf(temp, "%.2f", frequency);
fl_set_input(fdui -> main_form_frequency_input_field, temp);

sprintf(temp, "%c", polarization);
fl_set_input(fdui -> main_form_polarization_input_field, temp);

sprintf(temp, "%d", symbolrate);
fl_set_input(fdui -> main_form_symbolrate_input_field, temp);

fl_set_input(fdui -> main_form_angle_input_field, "");

fl_set_input(fdui -> main_form_satellite_location_input_field, position);

fl_set_input(fdui -> main_form_satellite_name_input_field, "hotbird");

fl_set_object_label(fdui -> main_form_output_filename_output_field, output_dir);

fl_set_input(fdui -> main_form_pids_input_field, pids_string);

if( load_programs() )
	{
//	show_programs(SHOW_TOP);
	show_programs(SHOW_SAME_POSITION);
	}

load_local_position(); // this may fail if not yet set

load_timers();

/* restore search state */
load_search_settings();

load_fontsizes();

/* fontstyle does not work in streambrowser ??? */
//a = fl_set_font_name(16, "-adobe-courier-bold-o-normal--24-240-75-75-m-150-iso8859-10");
//a = fl_set_font_name(17, "-adobe-helvetica-medium-r-*-*-*-?-*-*-*-*-iso8859-1");
//a = fl_set_font_name(17, "-adobe-helvetica-bold-r-*-*-*-?-*-*-*-*-iso8859-1");
//fl_set_browser_fontstyle(fdui -> main_form_stream_browser, a);
//fl_redraw_form(fdui -> main_form);
//XSync(fl_get_display(), 0);
/* refresh the X screen to show the new fonts */
//FILE *pptr;
//sprintf(temp, "xrefresh");
//pptr = popen(temp, "w");
//pclose(pptr);

/* restore selection in stream browser */
ptr = load_setting("selected_stream");
if(ptr)
	{
	b = sscanf(ptr, "%d", &a);
	if( select_stream(a) && (b == 1) )
		{ 
		selected_stream = a;

		fl_select_browser_line(fdui -> main_form_stream_browser, selected_stream);
		/* make sure selected line is visible, and somewhere in the middle of the browser */
		if(selected_stream > 2) fl_set_browser_topline(fdui -> main_form_stream_browser, selected_stream - 2);
		free(ptr);
		}

	/* NOTE: Terratec cinergy S2 USB this causes ioctl to frontend and a firmware upload */
	/* and move the dish to the sat or angle */
	main_form_goto_button_cb(0, 0);            
	}

/* beep mode */
ptr = load_setting("beep_mode");
if(ptr)
	{
	a = sscanf(ptr, "%d", &b);
	if(a == 1) dsp_beep_flag = b;
	fl_set_button(fdui -> main_form_beep_select_button, dsp_beep_flag);
	}


fl_set_button(fdui -> main_form_enable_remote_button, enable_remote_flag);

fl_do_forms();

exit(0);
} /* end function main */

